'''7836685 kg956 CSCI935  Karan Goel Assignment 3'''

import sys
import os
import cv2
import numpy as np

def iou(boxA, boxB):
    '''Compute the intersection over union of two boxes'''
    xA = max(boxA[0], boxB[0])
    yA = max(boxA[1], boxB[1])
    xB = min(boxA[2], boxB[2])
    yB = min(boxA[3], boxB[3])
    
    interArea = max(0, xB - xA) * max(0, yB - yA)
    boxAArea = (boxA[2] - boxA[0]) * (boxA[3] - boxA[1])
    boxBArea = (boxB[2] - boxB[0]) * (boxB[3] - boxB[1])
    
    iou = interArea / float(boxAArea + boxBArea - interArea)
    
    return iou



def detect_objects_cnn(frame):
# Loading model
 w = 300
 h = 300
 model = cv2.dnn.readNetFromTensorflow('frozen_inference_graph.pb',
                                      'ssd_mobilenet_v2_coco_2018_03_29.pbtxt.txt')
 
 input_size = (w, h)
 resized_frame = cv2.resize(frame, input_size)
 detect_frame = resized_frame.copy()
 

 model.setInput(cv2.dnn.blobFromImage(resized_frame, size=(300, 300), swapRB=True))
 output = model.forward()

 current_boxes = []
 previous_boxes = []
 labels = []

 for detection in output[0, 0, :, :]:
    confidence = detection[2]
    class_id = detection[1]
    if confidence > .4 and class_id == 1:
        box_x = detection[3] * w
        box_y = detection[4] * h
        box_width = detection[5] * w
        box_height = detection[6] * h
        current_boxes.append([int(box_x), int(box_y), int(box_width), int(box_height)])
        cv2.rectangle(detect_frame, (int(box_x), int(box_y)), (int(box_width), int(box_height)), (23, 230, 210), thickness=1)

 new_labels = [-1] * len(current_boxes)
 nextLabel = 1
 for i, box in enumerate(current_boxes):
    for j, prev_box in enumerate(previous_boxes):
            if iou(box, prev_box) > 0.5:
                new_labels[i] = labels[j]
                break
    if new_labels[i] == -1:
            new_labels[i] = nextLabel
            nextLabel += 1

 for i, box in enumerate(current_boxes):
        startX, startY, endX, endY = box
        cv2.rectangle(frame, (startX, startY), (endX, endY), (0, 255, 0), 2)
        cv2.putText(frame, str(new_labels[i]), (startX, startY-10), cv2.FONT_HERSHEY_SIMPLEX, 0.5, (0, 255, 0), 2)

 previous_boxes = current_boxes
 labels = new_labels

#  display_frame = np.vstack([
#         np.hstack([resized_frame, detect_frame]),
#         np.hstack([resized_frame, resized_frame])
#  ])
 
 
 
 return frame


def detect_objects(frame, frame_count, bg_subtractor):
    fgmask = bg_subtractor.apply(frame,  learningRate=-1)
    bg_only = bg_subtractor.getBackgroundImage()
    

    # Detect moving pixels using Gaussian Mixture background modelling
    fgmask_raw = fgmask.copy()  # Before noise removal

    # Remove noise using morphological operators
    kernel = cv2.getStructuringElement(cv2.MORPH_ELLIPSE, (5, 5))
    fgmask = cv2.morphologyEx(fgmask, cv2.MORPH_CLOSE, kernel)
    fgmask = cv2.morphologyEx(fgmask, cv2.MORPH_OPEN, kernel)

    # Connected component analysis
    ret, labels = cv2.connectedComponents(fgmask)

    # Output buffer for detected objects
    detected = np.zeros_like(frame)

    # Classify each object and count them
    person_count, car_count, other_count = 0, 0, 0
    for label in range(1, ret):
        component = (labels == label).astype(np.uint8) * 255
        contours, _ = cv2.findContours(component, cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_SIMPLE)
        for contour in contours:
            x, y, w, h = cv2.boundingRect(contour)
            aspect_ratio = float(w) / h
            if 0.3 < aspect_ratio < 0.7:
                person_count += 1
            elif aspect_ratio > 1.5:
                car_count += 1
            else:
                other_count += 1
            detected[y:y+h, x:x+w] = frame[y:y+h, x:x+w]

    total_objects = person_count + car_count + other_count
    print(f"Frame {frame_count:04d}: {total_objects} objects ({person_count} persons, {car_count} cars and {other_count} others)")

    #Stack all frames for display
    fgmask_raw = cv2.cvtColor(fgmask_raw, cv2.COLOR_GRAY2BGR)
    display_frame = np.vstack([
        np.hstack([frame, bg_only]),
        np.hstack([fgmask_raw, detected])
    ])

    return display_frame

def main(*args):
    """
    Main execution function.
    """
    flag = args[0]
    path = args[1]

    #vidture video
    vid = cv2.VideoCapture(path)

    # Check if the video file opened successfully
    if not vid.isOpened():
     print("Error: Couldn't open the video.")
     exit()

    width = int(vid.get(3))
    height = int(vid.get(4))
    scale = 640.0 / width
    vid.set(3, 640)
    vid.set(4, int(height * scale))

    frame_count = 0
    bg_subtractor = cv2.createBackgroundSubtractorMOG2(history=500, varThreshold=50, detectShadows=True)

    while vid.isOpened():
        ret, frame = vid.read()
        if not ret:
            break
        frame_count += 1

        #Task 1
        if flag == '-b':
          output_frame = detect_objects(frame, frame_count, bg_subtractor)
          cv2.imshow('Task 1', output_frame)
          if cv2.waitKey(1) & 0xFF == ord('q'):
            break

        #Task 2
        elif flag == '-d':
          output_frame = detect_objects_cnn(frame)
          cv2.imshow('Task 2', output_frame)
          if cv2.waitKey(1) & 0xFF == ord('q'):
            break

        else:
          print("flag unspecified")
          break

    vid.release()
    cv2.destroyAllWindows()


# get args from user
if __name__ == "__main__":
    if len(sys.argv) < 3:
        print("Insufficient arguments provided.")
        exit(0)

    main(*sys.argv[1:])